@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## HTML Object - ~/lib/HTML/Object/Element.pm
-## Version v0.2.3
+## Version v0.2.4
 ## Copyright(c) 2022 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2021/04/25
-## Modified 2022/11/12
+## Modified 2023/05/11
 ## All rights reserved
 ## 
 ## 
@@ -35,7 +35,7 @@ BEGIN
     our $LOOK_LIKE_HTML = qr/^[[:blank:]\h]*\<\w+.*?\>/;
     our $LOOK_LIKE_IT_HAS_HTML = qr/\<\w+.*?\>/;
     our $ATTRIBUTE_NAME_RE = qr/\w[\w\-]*/;
-    our $VERSION = 'v0.2.3';
+    our $VERSION = 'v0.2.4';
 };
 
 use strict;
@@ -620,6 +620,7 @@ sub extract_links
     my $has_expectation = scalar( keys( %$wants ) );
     my $a = $self->new_array;
     my $crawl;
+    my $seen = {};
     $crawl = sub
     {
         my $kids = shift( @_ );
@@ -627,22 +628,39 @@ sub extract_links
         {
             my $e = shift( @_ );
             my $def;
-            $def = $HTML::Object::LINK_ELEMENTS->{ $e->tag } if( exists( $HTML::Object::LINK_ELEMENTS->{ $e->tag } ) );
-            return(1) if( !defined( $def ) );
-            return(1) if( $has_expectation && !exists( $wants->{ $e->tag } ) );
-            foreach my $attr ( @$def )
+            my $tag = $e->tag;
+            $self->debug(4);
+            $def = $HTML::Object::LINK_ELEMENTS->{ "$tag" } if( exists( $HTML::Object::LINK_ELEMENTS->{ "$tag" } ) );
+            $self->debug(0);
+            # return(1) if( !defined( $def ) );
+            # return(1) if( $has_expectation && !exists( $wants->{ "$tag" } ) );
+            if( defined( $def ) && 
+                (
+                    !$has_expectation || 
+                    ( $has_expectation && !exists( $wants->{ "$tag" } ) )
+                ) )
             {
-                my $val;
-                if( $e->attributes->exists( $attr ) && length( $val = $e->attributes->get( $attr ) ) )
+                foreach my $attr ( @$def )
                 {
-                    $a->push( $self->new_hash({
-                        attribute => $attr,
-                        element   => $e,
-                        tag       => $e->tag,
-                        value     => $val,
-                    }) );
+                    my $val;
+                    if( $e->attributes->exists( $attr ) && length( $val = $e->attributes->get( $attr ) ) )
+                    {
+                        $a->push( $self->new_hash({
+                            attribute => $attr,
+                            element   => $e,
+                            tag       => $tag,
+                            value     => $val,
+                        }) );
+                    }
                 }
             }
+            my $addr = Scalar::Util::refaddr( $e );
+            if( ++$seen->{ $addr } > 1 )
+            {
+                return(1);
+            }
+            $crawl->( $e->children );
+            return(1);
         });
     };
     $crawl->( $self->children );
@@ -1611,7 +1629,7 @@ HTML::Object::Element - HTML Element Object
 
 =head1 VERSION
 
-    v0.2.3
+    v0.2.4
 
 =head1 DESCRIPTION
 
@@ -1847,20 +1865,43 @@ Actually, I am not sure this should be here, and rather it should be in L<HTML::
 
 =head2 extract_links
 
-Returns an L<array object|Module::Generic::Array> containing L<hash objects|Module::Generic::Hash>, for each attribute of an element containing a link, with the following properties:
+Returns links found by traversing the element and all of its children and looking for attributes (like C<href> in an C<<a>> element, or C<src> in an C<<img>> element) whose values represent links.
+
+You may specify that you want to extract links from just some kinds of elements (instead of the default, which is to extract links from all the kinds of elements known to have attributes whose values represent links). For instance, if you want to extract links from only C<<a>> and C<<img>> elements, you could code it like this:
+
+    my $links = $elem->extract_links( qw( a img ) ) ||
+        die( $elem->error );
+    foreach( @$links )
+    {
+        say "Hey, there is a ", $_->{tag}, " that links to ", $_->{value}, "in its ", $_->{attribute}, " attribute, at ", $_->{element}->address;
+    }
+
+The dictionary definition hash reference of all tags and their attributes containing potential links is available as C<$HTML::Object::LINK_ELEMENTS>
+
+This method returns an L<array object|Module::Generic::Array> containing L<hash objects|Module::Generic::Hash>, for each attribute of an element containing a link, with the following properties:
 
 =over 4
 
-=item I<attribute>
+=item * C<attribute>
 
-=item I<element>
+The attribute containing the link
 
-=item I<tag>
+=item * C<element>
 
-=item I<value>
+The L<element object|HTML::Object::Element>
+
+=item * C<tag>
+
+The element tag name.
+
+=item * C<value>
+
+The attribute value, which would typically contain the link value.
 
 =back
 
+Nota bene: this method has been implemented to provide similar API as L<HTML::Element> and the 2 first paragraphs of this method description are taken from this module.
+
 =head2 find_by_attribute
 
 Returns an L<array object|Module::Generic::Array> of all the elements (including potentially the current element itself) in the element's hierarchy who have an attribute that matches the given attribute name.
